@@ -2,7 +2,6 @@ use rayon::prelude::*;
 use std::collections::{HashMap, HashSet};
 //Imports Arc:
 use std::sync::{Arc, Mutex};
-
 #[derive(Debug, PartialEq, Clone)]
 pub struct Sentence {
     pub index: usize,
@@ -62,7 +61,18 @@ impl<'a> Summariser<'a> {
                 let mut words: HashSet<String> = HashSet::new();
                 if !ngrams {
                     words = HashSet::from_iter(
-                        sentence.split_whitespace().map(|word| word.to_string()),
+                        sentence
+                        //.chars()
+                        //.collect::<Vec<char>>()
+                        //.iter()
+                        //.map(|x| if !x.is_alphanumeric() || !x.is_ascii_whitespace() {
+                            //let spaced_char = format!(" {} ", x);
+                            //spaced_char.to_string().to_lowercase()
+                        //} else {
+                            //x.to_string().to_lowercase()
+                        //})
+                        //.collect::<String>()
+                        .split_whitespace().map(|word| word.to_string()),
                     );
                 } else {
                     for n in 7..15 {
@@ -101,6 +111,57 @@ impl<'a> Summariser<'a> {
         }
     }
 
+    pub fn score_sentences_by_word_frequency(
+        &mut self,
+        word_frequencies: HashMap<String, f32>,
+        density: f32,
+        length_penalty: f32
+    ) -> Vec<(usize, f32)> {
+        for sentence in self.sentences.values_mut() {
+            let mut score = 0.0;
+            for word in sentence.words.iter() {
+                let frequency = word_frequencies.get(word).unwrap_or(&-1.0);
+                //let mut score = sentence.number_of_connections;
+                score += frequency;
+            }
+            sentence.number_of_connections = score.powf(density)
+            / (sentence.length as f32).powf(length_penalty);
+        }
+        //Sort the sentences by score
+        let mut sorted_sentences: Vec<(usize, f32)> = self
+            .sentences
+            .iter()
+            .map(|(index, sentence)| (*index, sentence.number_of_connections))
+            .collect();
+        sorted_sentences.sort_by(|a, b| b.1.partial_cmp(&a.1).unwrap());
+        return sorted_sentences;
+    }
+
+    pub fn approximate_top_sentences(&mut self, 
+        k: usize,
+        density: f32,
+        length_penalty: f32
+    ) -> Vec<Sentence> {
+        let mut word_frequencies: HashMap<String, f32> = HashMap::new();
+        for sentence in self.sentences.values() {
+            for word in sentence.words.iter() {
+                let count = word_frequencies.entry(word.clone()).or_insert(-1.0);
+                *count += 1.0;
+            }
+        }
+        //for (word, score) in word_frequencies.iter_mut() {
+            //*score = score.powf(3.0);
+        //}
+        let sorted_sentences = self.score_sentences_by_word_frequency(word_frequencies, density, length_penalty);
+        let top_sentences_text = sorted_sentences
+            .iter()
+            //.rev()
+            .map(|(indx, _)| self.sentences.get(indx).unwrap().clone())
+            .take(k)
+            .collect::<Vec<Sentence>>();
+        top_sentences_text
+    }
+
     pub fn retrieve_sentence_by_index(&self, index: usize) -> Option<Sentence> {
         self.sentences.get(&index).cloned()
     }
@@ -406,7 +467,7 @@ impl<'a> Summariser<'a> {
         //let sentence_b = self.sentences.get(&sentence_b_indx).unwrap();
         //However, the index might not exist in the HashMap, so we need to check for that
         //THIS IS THE BUG, IT NEEDS TO BE 0
-        //let mut overlapping_words_with_b_length = 1;
+        let mut overlapping_words_with_b_length = 0;
         if let Some(sentence_a) = self.sentences.get(&sentence_a_indx) {
             if let Some(sentence_b) = self.sentences.get(&sentence_b_indx) {
                 let intersection_length = sentence_a
@@ -415,16 +476,16 @@ impl<'a> Summariser<'a> {
                     .collect::<Vec<_>>()
                     .len();
                 //.map(|x| x.to_string())
-                //if self.bias_list.len() > 0 {
-                //overlapping_words_with_b_length = self.bias_list
-                //.intersection(&sentence_b.words)
-                //.collect::<Vec<_>>()
-                //.len() //+ self.bias_list
+                if self.bias_list.len() > 0 {
+                overlapping_words_with_b_length = self.bias_list
+                .intersection(&sentence_b.words)
+                .collect::<Vec<_>>()
+                .len(); //+ self.bias_list
                 //.intersection(&sentence_a.words)
                 //.collect::<Vec<_>>()
                 //.len();
-                //}
-                return intersection_length as f32; // * (1.0 + ((overlapping_words_with_b_length as f32 * 3.0).powf(self.bias_strength.unwrap()) / (sentence_a.length as f32).powf(0.64)));
+                }
+                return intersection_length as f32 * (1.0 + ((overlapping_words_with_b_length as f32 * 3.0).powf(self.bias_strength.unwrap()) / (sentence_a.length as f32).powf(0.64)));
             } else {
                 return 0.0;
             }
@@ -460,6 +521,14 @@ fn main() {
             The file pithy will read from. Required.
         --sentences:
             The number of sentences for pithy to return. Defaults to 3.
+        --approximate:
+            Will return a decent approximation of the summary. Good
+            for extremely long texts where you don't care about precision.
+        --bias:
+            slash (i.e \"/\") separated list of words to bias the summary towards.
+            If you are using pithy on a large text, increase the chunk_size to
+            2500-5000 to get relevant results. Note that this doesn't work in
+            approximate mode.
         --bias_strength:
             The strength of the bias, must be an integer. Defaults to 6.
         --by_section:
@@ -523,6 +592,7 @@ fn main() {
         return;
     }
     let by_section = arguments.contains(&"--by_section".to_string());
+    let approximate = arguments.contains(&"--approximate".to_string());
     let chunk_size = if arguments.contains(&"--chunk_size".to_string()) {
         Some(
             arguments
@@ -561,7 +631,7 @@ fn main() {
                 .unwrap(),
         )
     } else {
-        Some(2.0)
+        Some(6.0)
     };
     let filename = arguments
         .get(arguments.iter().position(|x| x == "-f").unwrap() + 1)
@@ -658,18 +728,24 @@ fn main() {
     }
     //summariser.clean_sentences(clean_whitespace, clean_nonalphabetic, clean_caps);
     //api: number_of_sentences_to_return: usize, return_summaries_for_each: bool, chunk_size: Option<usize>, force_sum_all: bool, length_penalty: f32
-    let mut summary = summariser.top_sentences(
-        number_of_sentences_to_return,
-        by_section,
-        chunk_size,
-        force_all,
-        length_penalty,
-        force_chunk,
-        density,
-        bias_list,
-        bias_strength,
-        !use_bar,
-    );
+        let mut summary = if !approximate {
+            let sum = summariser.top_sentences(
+            number_of_sentences_to_return,
+            by_section,
+            chunk_size,
+            force_all,
+            length_penalty,
+            force_chunk,
+            density,
+            bias_list,
+            bias_strength,
+            !use_bar,
+        );
+        sum
+    } else {
+        let sum = summariser.approximate_top_sentences(number_of_sentences_to_return, density, length_penalty);
+        sum
+    };
     if !use_bar {
         println!("Summarising took {} seconds", now.elapsed().as_secs_f32());
     }
